import os
import re
import json
import requests
import time
import pathlib
import fpdf
import base64
import datetime
import sqlite3

template_opprett = pathlib.Path(__file__).with_name("template_opprett.xml")
template_hentsak = pathlib.Path(__file__).with_name("template_hentsak.xml")
template_endrestatus = pathlib.Path(__file__).with_name("template_endrestatus.xml")
template_leggtilfil = pathlib.Path(__file__).with_name("template_leggtilfil.xml")
debug_values = pathlib.Path(__file__).with_name("debug.json")

settings = {}
oids = []

def send_request(xml: str):
    headers = {
                'Content-Type': 'application/soap+xml;charset=UTF-8;action="urn:no:vegvesen:saksarkiv:journalpost:tjeneste:journalpost-1/JournalpostSoapPortType/opprett"'
                }
    login = requests.auth.HTTPBasicAuth(settings['MIME_USER'],settings['MIME_PWD'])
    response = requests.request("POST", settings['MIME_ENDPOINT_OPPRETT'], headers=headers, data=xml.encode("utf-8"), auth=login)
    print(response.text)
    resultat = re.search('<DokumentId>(.*)<\/DokumentId>', response.text)
    dokid = str(resultat.group(1))
    if "<" in dokid:
        print("\t\t\tDokumentID: " + dokid.split("<")[0])
        return dokid.split("<")[0]
    print("\t\t\tDokumentID: " + dokid)
    return dokid

def legg_til_vedlegg(fil, dokumentid):
    with open(fil, 'rb') as innhold:
        fil64 = base64.b64encode(innhold.read()).decode('utf-8')
    filnavn = pathlib.Path(fil)
    p = {}
    p["filinnhold"] = fil64
    p["filtype"] = filnavn.suffix[1:].lower()
    p["dokumentid"] = dokumentid
    p['mimebruker'] = settings["MIME_USER"]
    xml = apply_params(p, template_leggtilfil)
    headers = {
                'Content-Type': 'application/soap+xml;charset=UTF-8;action="urn:no:vegvesen:saksarkiv:journalpost:tjeneste:journalpost-1/JournalpostSoapPortType/leggTilNyttDokument"'
                }
    login = requests.auth.HTTPBasicAuth(settings['MIME_USER'],settings['MIME_PWD'])
    response = requests.request("POST", settings['MIME_ENDPOINT_OPPRETT'], headers=headers, data=xml.encode("utf-8"), auth=login)
    #print(xml.encode("utf-8"))
    print("\t\t\tLa til vedlegg: " + filnavn.name)
    print(response.text)

def get_settings(prosjektnavn, sqlitedb, featureLayerUrl, saksnummer, mime_user, mime_pwd, felt_navn, felt_mail, felt_innspill,opprett_url="https://www.test.vegvesen.no/ws/no/vegvesen/sakarkiv/journalpost", hentsak_url="https://www.test.vegvesen.no/ws/no/vegvesen/sakarkiv/saksmappe"):
    settings["MIME_USER"] = mime_user
    settings["MIME_PWD"] = mime_pwd
    settings["MIME_ENDPOINT_OPPRETT"] = opprett_url
    settings["MIME_ENDPOINT_HENTSAK"] = hentsak_url
    settings["prosjektnavn"] = prosjektnavn
    settings["featureLayer"] = featureLayerUrl
    settings["sqlitedb"] = sqlitedb
    settings["saksnummer"] = saksnummer
    settings["felt_navn"] = felt_navn
    settings["felt_epost"] = felt_mail
    settings["felt_innspill"] = felt_innspill

def check_attachments(flayerUrl: str, oid: str):
    vedlegg = []
    f = requests.get(flayerUrl + f"/queryAttachments?objectIds={oid}&returnUrl=true&f=json")
    atts = f.json()
    #print(atts)
    if "attachmentGroups" in atts.keys():
        for gr in atts["attachmentGroups"]:
            for att in gr['attachmentInfos']:
                v = requests.get(att['url'])
                with open(att['name'], 'wb') as fil:
                    fil.write(v.content)
                    vedlegg.append(att['name'])
                    #print(f"La til " + att['name'])
        #print(vedlegg)
    return vedlegg

def get_debug_values():
    with open(debug_values, "r") as r:
        v = json.load(r)
    return v

def apply_params(params, templatesti):
    params['mottattDato'] = time.strftime("%Y-%m-%d")     # Dato for når innspillet blir sendt til MIME
    params['dokumentDato'] = time.strftime("%Y-%m-%d")    # Dato for når innspillet ble lagret i Innbyggerdialog

    with open(templatesti, "r", encoding="utf-8") as fil:
        xml = fil.read()
    vars = re.findall("{.[a-zA-Z]*}", xml)
    for v in vars:
        #print(v)
        if v[1:-1] in params.keys():
            if not len(params[v[1:-1]]) > 100:
                print("\t\t\tErstatter {0} -> {1}".format(v, params[v[1:-1]]))
                #print(params[v[1:-1]])
            xml = xml.replace(v, params[v[1:-1]])
    xml = xml.replace("$", "")
    return xml

def text_to_pdf(innspill, navn, dato, i):
    filnavn = f"Innspill fra {navn}.pdf"
    print(f"\t\tOppretter {navn}.pdf")
    pdf = fpdf.FPDF()
    pdf.add_page()
    pdf.set_font('Arial', 'B', 14)
    pdf.cell(40, 10, "Fra:", ln = 0)
    pdf.set_font('Arial', size=14)
    pdf.cell(0, 10, navn, ln = 1)
    pdf.set_font('Arial', 'B', 14)
    pdf.cell(40, 10, "Dato:", ln = 0)
    pdf.set_font('Arial', size=14)
    pdf.cell(0, 10, dato, ln = 1)
    pdf.set_font('Arial', 'B', 14)
    pdf.cell(40, 10, "Innspill/melding:", ln = 0)
    pdf.set_font('Arial', size=14)
    pdf.cell(0, 10, innspill, ln = 1)
    pdf.ln()
    for k in i.keys():
        pdf.set_font('Arial', 'B', 10)
        pdf.cell(40, 10, str(k) + ":", ln = 0)
        pdf.set_font('Arial', size=10)
        pdf.cell(0, 10, str(i[k]), ln = 1)

    pdf.output(filnavn, 'F')
    #print("\tPDF opprettet")
    return filnavn

def endre_status(dokumentid: str):
    headers = {
                'Content-Type': 'application/soap+xml;charset=UTF-8;action="urn:no:vegvesen:saksarkiv:journalpost:tjeneste:journalpost-1/JournalpostSoapPortType/endreStatus"'
                }
    i = {}
    i['dokumentid'] = dokumentid
    i['mimebruker'] = settings["MIME_USER"]
    xml = apply_params(i, template_endrestatus)
    login = requests.auth.HTTPBasicAuth(settings['MIME_USER'],settings['MIME_PWD'])
    #print(xml)
    response = requests.request("POST", settings["MIME_ENDPOINT_OPPRETT"], headers=headers, data=xml.encode("utf-8"), auth=login)
    print(response.text)

def read_features(flayerUrl: str):
    #layerColl = arcgis.features.FeatureLayer(flayerUrl)
    #featureSet = layerColl.query(where="1=1")
    f = requests.get(flayerUrl + "/query?where=1%3D1&outFields=*&returnGeometry=true&f=json")
    featureSet = f.json()
    if "error" in featureSet.keys():
        print("\n\nFikk ikke tilgang til Feature Layer\n\n")
        print(featureSet)
    oidfelt = featureSet['objectIdFieldName']
    innspill = []
    for feat in featureSet["features"]:
        if feat["attributes"][oidfelt] in oids:
            print("\tDette objektet har allerede blitt prosessert: {0}".format(feat["attributes"][oidfelt]))
            continue
        else:
            print("\tFant nytt innspill")
            vedlegg = []
            i = {}
            if settings["felt_navn"] in feat["attributes"].keys():
                i['navn'] = feat["attributes"][settings["felt_navn"]]
            else:
                i['navn'] = ""
            if settings["felt_epost"] in feat["attributes"].keys():
                #print("Fant epostfelt")
                i['mail'] = feat["attributes"][settings["felt_epost"]]
                #print(i['mail'])
            else:
                i['mail'] = "" 
            if settings["felt_innspill"] in feat["attributes"].keys():
                i['innspill'] = feat["attributes"][settings["felt_innspill"]]
            else:
                i['innspill'] = ""
            i['dato'] = datetime.datetime.fromtimestamp(feat["attributes"]["CreationDate"]/1000).strftime('%Y-%m-%d')
            attachments = check_attachments(flayerUrl, feat["attributes"][oidfelt])
            pdf = text_to_pdf(i['innspill'], i['navn'], i['dato'], feat["attributes"])
            with open(pdf, 'rb') as innhold:
                pdfinnhold = base64.b64encode(innhold.read()).decode('utf-8')
            i['pdfinnhold'] = pdfinnhold 
            #attachments.append(pdf)
            i['vedlegg'] = attachments
            i['oid'] = feat["attributes"][oidfelt]
            i["prosjektnavn"] = settings["prosjektnavn"]
            i["mimebruker"] = settings["MIME_USER"]
            innspill.append(i)
            print("\tLa til innspill fra {0} med {1} vedlegg".format(i['navn'], len(attachments)))
            #oids.append(feat["attributes"][oidfelt])
    return innspill

def hent_sak(saksnummer: str):
    headers = {
                'Content-Type': 'application/soap+xml;charset=UTF-8;action="urn:no:vegvesen:saksarkiv:saksmappe:tjeneste:saksmappe-1/SaksmappeSoapPortType/hent"'
                }
    i = {}
    i['saksnummer'] = saksnummer
    i['mimebruker'] = settings["MIME_USER"]
    xml = apply_params(i, template_hentsak)
    login = requests.auth.HTTPBasicAuth(settings['MIME_USER'],settings['MIME_PWD'])
    print(settings['MIME_ENDPOINT_HENTSAK'])
    response = requests.request("POST", settings['MIME_ENDPOINT_HENTSAK'], headers=headers, data=xml.encode("utf-8"), auth=login)
    print(response.text)
    resultat = re.search("<Ansvarskode>(.*)</Ansvarskode>", response.text)
    print("\t\t\tAnsvarlig enhet: {0}".format(resultat.group(1)))
    return resultat.group(1)

def hent_journalforte(saksnr: str, dbsti: str):
    sjekk_fil = pathlib.Path(dbsti)
    if sjekk_fil.is_file():
        print("\t\t\tFant database")
    else:
        print("\t\t\tOBS! Fant ikke SQLITE-database")
    conn = sqlite3.connect(dbsti)
    cur = conn.cursor()
    sql = "SELECT objectid FROM henvendelser WHERE saksnummer = '{0}'".format(saksnr)
    print("\t\t\t\tSQL: "+ sql)
    cur.execute(sql)
    rows = cur.fetchall()
    for row in rows:
        oids.append(row[0])
    conn.close()

def registrer_journalfort(saksnr: str, objectid: int, dbsti: str):
    conn = sqlite3.connect(dbsti)
    cur = conn.cursor()
    sql = "INSERT INTO henvendelser VALUES('{0}', {1})".format(saksnr, objectid)
    cur.execute(sql)
    conn.commit()
    conn.close()

def main():
    print("Sjekker for innspill på sak {0}".format(settings["saksnummer"]))
    oids = hent_journalforte(settings["saksnummer"], settings["sqlitedb"])
    innspill = read_features(settings["featureLayer"])
    if len(innspill) > 0:
        for i in innspill:
            #print(i)
            i["saksnummer"] = settings["saksnummer"]            
            i["prosjektnavn"] = settings["prosjektnavn"]
            db_verdier = get_debug_values()
            for k in db_verdier.keys():
                if not k in i.keys():
                    i[k] = db_verdier[k]
            #i['filbase64'] = zip_it(i['vedlegg'])
            #i['vedlegg'] = None
            print("\t\tHenter registrert sak i MIME")
            i['ansvarligEnhet'] = hent_sak(settings["saksnummer"])
            print("\t\t\tOppretter XML")
            xml = apply_params(i, template_opprett)
            dokid = send_request(xml)
            print("\t\t\tInnspill registert i MIME")
            #print("\t\tDokumentID: " + dokid)
            print("\t\t\tLegger til vedlegg")
            for fil in i['vedlegg']:
                legg_til_vedlegg(fil, dokid)
            endre_status(dokid)
            print("\t\tStatus satt til journalført")
            registrer_journalfort(settings["saksnummer"], i['oid'], settings["sqlitedb"])
    else:
        print("Ingen nye innspill registrert")

